(**
 ** $Id: Convolve.sml 91 2005-12-16 17:15:50Z rlpm $
 **
 ** Convolution helper for the image processing library for Standard ML
 ** Author: Rory McGuire <rlpm at cs dot unm dot edu>
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
 ** published by the Free Software Foundation; either version 2 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
 **
 ** You can obtain a copy of the GPL at:
 **   http://www.fsf.org/copyleft/gpl.html.
 **)

signature ARITH =
sig
    type a
    val zero : a
    val + : a * a -> a
    val * : a * a -> a
end

functor Convolve (A:ARITH) =
struct
fun oneDim g f =
    let
        val flen = Vector.length f
        val glen = Vector.length g
    in
        if flen < glen
        then oneDim f g
        else
            let
                fun fref t = Vector.sub(f, t mod flen)
                val m = Int.-(glen,1) div 2
            in
                Vector.tabulate
                    (flen,
                     (fn t =>
                         Vector.foldl
                             A.+
                             A.zero
                             (Vector.tabulate
                                  (glen,
                                   (fn y =>
                                       let
                                           val x = Int.-(y,m)
                                       in
                                           A.*(fref (Int.+(t,x)),
                                               Vector.sub
                                                   (g,Int.-(Int.-(glen,y),
                                                            1)))
                                       end)))))
            end
    end

fun twoDim g f =
    let
        val frows = Array2.nRows f
        val fcols = Array2.nCols f
        val grows = Array2.nRows g
        val gcols = Array2.nCols g
        val _ = if frows < gcols orelse fcols < gcols
                then raise Subscript
                else ()
        fun fref (x,y) = Array2.sub(f,x mod frows,y mod fcols)
        val mrows = Int.-(grows,1) div 2
        val mcols = Int.-(gcols,1) div 2
    in
        Array2.tabulate
            Array2.RowMajor
            (frows,fcols,
             (fn (tx,ty) =>
                 Array2.fold
                     Array2.RowMajor
                     A.+
                     A.zero
                     (Array2.tabulate
                          Array2.RowMajor
                          (grows,gcols,
                           (fn (x,y) =>
                               let
                                   val xx = Int.-(x,mrows)
                                   val yy = Int.-(y,mcols)
                               in
                                   A.*(fref(Int.+(tx,xx),
                                            Int.+(ty,yy)),
                                       Array2.sub
                                           (g,
                                            Int.-(Int.-(grows,x),1),
                                            Int.-(Int.-(gcols,y),1)))
                               end)))))
    end
end
