(**
 ** $Id: ImageColor.sig 95 2005-12-16 18:30:44Z rlpm $
 **
 ** An image processing library for Standard ML
 ** Author: Rory McGuire <rlpm at cs dot unm dot edu>
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
 ** published by the Free Software Foundation; either version 2 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
 **
 ** You can obtain a copy of the GPL at:
 **   http://www.fsf.org/copyleft/gpl.html.
 **)

signature IMAGE_COLOR =
sig
  type image = (real * real * real) Image_Image.image

  val fromGrey : ImageGrey.image -> image

  val red : image -> ImageGrey.image
  val green : image -> ImageGrey.image
  val blue : image -> ImageGrey.image

  val toRGB : image -> {red:ImageGrey.image,
                        green:ImageGrey.image,
                        blue:ImageGrey.image}
  val fromRGB : {red : ImageGrey.image,
                 green : ImageGrey.image,
                 blue : ImageGrey.image} -> image

  val fromGreyHot : ImageGrey.image -> image

  val fromHSV : {hue : ImageGrey.image,
                 saturation : ImageGrey.image,
                 value : ImageGrey.image} -> image

  val toHSV : image -> {hue : ImageGrey.image,
                        saturation : ImageGrey.image,
                        value : ImageGrey.image}

  (* map all values to real range (0.0,1.0) using (min,max) input *)
  val normalize : (real * real) -> image -> image

  (* map all values to real range (0.0,1.0) using min and max of values *)
  val maximizeContrast : image -> image

  (* write a PPM file *)
  val writePPMA : image * string -> unit
  val writePPM : image * string -> unit

  (* read PPM (ASCII or binary) files *)
  val readPPM : string -> image
end
