(**
 ** $Id: ImageComplex.sig 92 2005-12-16 17:34:14Z rlpm $
 **
 ** An image processing library for Standard ML
 ** Author: Rory McGuire <rlpm at cs dot unm dot edu>
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
 ** published by the Free Software Foundation; either version 2 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
 **
 ** You can obtain a copy of the GPL at:
 **   http://www.fsf.org/copyleft/gpl.html.
 **)

signature IMAGE_COMPLEX =
sig
  type image = Complex.complex Image_Image.image

  val fromGrey : ImageGrey.image -> image

  val fromRect : (ImageGrey.image * ImageGrey.image) -> image
  val toRect : image -> (ImageGrey.image * ImageGrey.image)

  val realPart : image -> ImageGrey.image
  val imagPart : image -> ImageGrey.image

  val fromPolar : (ImageGrey.image * ImageGrey.image) -> image
  val toPolar : image -> (ImageGrey.image * ImageGrey.image)

  val magnitude : image -> ImageGrey.image
  val phase : image -> ImageGrey.image

  val toColor : image -> ImageColor.image

  val + : image * image -> image
  val - : image * image -> image
  val * : image * image -> image
  val / : image * image -> image

  (* matrix-product of two images *)
  val product : image * image -> image

  val shrink : image * real -> image

  val convolveRows : Complex.complex vector -> image -> image
  val convolveCols : Complex.complex vector -> image -> image

  val convolve : Complex.complex Array2.array -> image -> image

  val dft : image -> image
  val idft : image -> image
  val fft : image -> image
  val ifft : image -> image
end
