(**
 ** $Id: ImageGrey.sig 93 2005-12-16 17:53:06Z rlpm $
 **
 ** An image processing library for Standard ML
 ** Author: Rory McGuire <rlpm at cs dot unm dot edu>
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
 ** published by the Free Software Foundation; either version 2 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
 **
 ** You can obtain a copy of the GPL at:
 **   http://www.fsf.org/copyleft/gpl.html.
 **)

signature IMAGE_GREY =
sig
  type image = real Image_Image.image

  (* map all values to real range (0.0,1.0) using (min,max) input *)
  val normalize : (real * real) -> image -> image

  (* map all values to real range (0.0,1.0) using min and max of values *)
  val maximizeContrast : image -> image

  (* write PGM files *)
  val writePGMA : image * string -> unit
  val writePGM : image * string -> unit

  (* read PGM (ASCII or binary) files *)
  val readPGM : string -> image

  val + : image * image -> image
  val - : image * image -> image
  val * : image * image -> image
  val / : image * image -> image

  (* matrix-product of two images *)
  val product : image * image -> image

  (* 256 grey level histogram *)
  val histogram : image -> int list

  (* 256 grey level joint histogram *)
  val jointHistogram : image * image -> int Array2.array

  val shrink : image * real -> image

  val convolveRows : real vector -> image -> image
  val convolveCols : real vector -> image -> image

  val convolve : real Array2.array -> image -> image
end
