(**
 ** $Id: Image_Image.sig 127 2006-02-14 18:44:00Z rlpm $
 **
 ** An image processing library for Standard ML
 ** Author: Rory McGuire <rlpm at cs dot unm dot edu>
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License as
 ** published by the Free Software Foundation; either version 2 of the
 ** License, or (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ** General Public License for more details.
 **
 ** You can obtain a copy of the GPL at:
 **   http://www.fsf.org/copyleft/gpl.html.
 **)

signature IMAGE_IMAGE =
sig
  type 'a image = 'a Array2.array

  val image : int * int * 'a -> 'a image

  val fromList : 'a list list -> 'a image
  val toList : 'a image -> 'a list list

  val tabulate : int * int * (int * int -> 'a) -> 'a image
  val tabulateFilter : int * int * (int * int -> 'a) -> 'a image

  val sub : 'a image * int * int -> 'a

  val nRows : 'a image -> int
  val nCols : 'a image -> int

  val row : 'a image * int -> 'a vector
  val column : 'a image * int -> 'a vector

  val mapi : (int * int * 'a -> 'b) -> 'a image -> 'b image
  val map : ('a -> 'b) -> 'a image -> 'b image

  val fold : ('a * 'b -> 'b) -> 'b -> 'a image -> 'b
  val foldi : (int * int * 'a * 'b -> 'b) -> 'b -> 'a image -> 'b

  val crop : ('a image * int * int * int * int) -> 'a image
  val pad : 'a -> ('a image * int * int) -> 'a image

  val binop : ('a * 'b -> 'c) -> 'a image * 'b image -> 'c image

  val transpose : 'a image -> 'a image

  val flip : 'a image -> 'a image
  val flop : 'a image -> 'a image

  val rotateCW : 'a image -> 'a image
  val rotateCCW : 'a image -> 'a image

  val downSampleRows : 'a image -> 'a image
  val downSampleCols : 'a image -> 'a image
  val downSample : 'a image -> 'a image

  val upSampleRows : 'a -> 'a image -> 'a image
  val upSampleCols : 'a -> 'a image -> 'a image
  val upSample : 'a -> 'a image -> 'a image

  (* concatenate images *)
  val leftToRight : 'a image list -> 'a image
  val topToBottom : 'a image list -> 'a image
end
